==> 0.0.3.tar.gz: lib/renderer/cache_renderer_mixin.dart <==
import 'dart:ui';

import 'package:flutter/foundation.dart';

import 'renderer.dart';

mixin CacheRendererMixin on Renderer {
  bool _waitingRefreshRender = true;
  bool _waitingRefreshChildren = true;
  Picture? _renderCache;
  Picture? _childrenCache;

  bool needCacheRender = true;
  bool needCacheChildren = true;

  @override
  void updateTree() {
    if (_waitingRefreshRender) update();
    if (_waitingRefreshChildren) updateChildren();
  }

  @override
  void renderTree(Canvas canvas) {
    final needRender = beforeRender();
    final needRenderChildren = beforeRenderChildren();

    if (!needRender && !needRenderChildren) return;

    transformCanvas(canvas, () {
      if (needRender) _renderWithCache(canvas);
      if (needRenderChildren) _renderChildrenWithCache(canvas);
    });
  }

  @mustCallSuper
  @override
  void refresh() {
    if (_waitingRefreshRender) return;

    _waitingRefreshRender = true;
    _renderCache = null;
    super.refresh();
  }

  @mustCallSuper
  @override
  void refreshChildren({bool fromChild = false}) {
    if (_waitingRefreshChildren) return;

    _waitingRefreshChildren = true;
    _childrenCache = null;
    super.refreshChildren(fromChild: fromChild);
  }

  void _renderChildrenWithCache(Canvas canvas) {
    if (_waitingRefreshChildren) {
      _waitingRefreshChildren = false;
      return renderChildren(canvas);
    }

    if (!needCacheChildren) {
      return renderChildren(canvas);
    }

    canvas.drawPicture(_childrenCache ??= _record(renderChildren));
  }

  void _renderWithCache(Canvas canvas) {
    if (_waitingRefreshRender) {
      _waitingRefreshRender = false;
      return render(canvas);
    }

    if (!needCacheRender) {
      return render(canvas);
    }

    canvas.drawPicture(_renderCache ??= _record(render));
  }

  Picture _record(void Function(Canvas) callback) {
    final recorder = PictureRecorder();
    callback(Canvas(recorder));
    return recorder.endRecording();
  }
}

==> 0.0.10.tar.gz: lib/renderer/cache_renderer_mixin.dart <==
import 'dart:ui';

import 'package:flutter/foundation.dart';

import 'renderer.dart';

mixin CacheRendererMixin on Renderer {
  bool _waitingRefreshRender = true;
  bool _waitingRefreshChildren = true;
  Picture? _renderCache;
  Picture? _childrenCache;

  bool needCacheRender = true;
  bool needCacheChildren = true;

  @override
  void updateTree() {
    if (_waitingRefreshRender) update();
    if (_waitingRefreshChildren) updateChildren();
  }

  @override
  void renderTree(Canvas canvas) {
    final needRender = beforeRender();
    final needRenderChildren = beforeRenderChildren();

    if (needRender || needRenderChildren) {
      transformCanvas(canvas, () {
        if (needRender) _renderWithCache(canvas);
        if (needRenderChildren) _renderChildrenWithCache(canvas);
      });
    }

    _waitingRefreshRender = false;
    _waitingRefreshChildren = false;
  }

  @mustCallSuper
  @override
  void tick() {
    if (_waitingRefreshRender) return;

    _waitingRefreshRender = true;
    _renderCache = null;
    super.tick();
  }

  @mustCallSuper
  @override
  void tickChildren({bool fromChild = false}) {
    if (_waitingRefreshChildren) return;

    _waitingRefreshChildren = true;
    _childrenCache = null;
    super.tickChildren(fromChild: fromChild);
  }

  void _renderChildrenWithCache(Canvas canvas) {
    if (_waitingRefreshChildren || !needCacheChildren) {
      return renderChildren(canvas);
    }

    canvas.drawPicture(_childrenCache ??= _record(renderChildren));
  }

  void _renderWithCache(Canvas canvas) {
    if (_waitingRefreshRender || !needCacheRender) {
      return render(canvas);
    }

    canvas.drawPicture(_renderCache ??= _record(render));
  }

  Picture _record(void Function(Canvas) callback) {
    final recorder = PictureRecorder();
    callback(Canvas(recorder));
    return recorder.endRecording();
  }
}
